#include "stdafx.h"
#include "hwdummy.h"
#include "mpxhwdummy.h"
#include <ctime>
#include <cassert>

using namespace std;

HwDummy::HwDummy(int hwID)
    : id(hwID)
    , callback(0)
    , version(2)
    , addNoise(true)
    , startTime(0)
    , elapsedTime(0)
{
    memset(lastErrStr, 0, MPXCTRL_MAX_MSGLENGTH * sizeof(char));
    acqPars.enableCst = FALSE;
    acqPars.mode = ACQMODE_ACQSTART_TIMERSTOP;
    acqPars.polarityPositive = TRUE;
    acqPars.time = 0;
    acqPars.useHwTimer = FALSE;

    devInfo.ifaceName = "dummy";
    devInfo.numberOfChips = 1;
    devInfo.numberOfRows = 256;
    devInfo.pixCount = MATRIX_SIZE;
    devInfo.rowLen = 256;
    devInfo.mpxType = MPX_ORIG;
    devInfo.clockReadout = devInfo.clockTimepix = 100;
    devInfo.timerMaxVal = 1e6;
    devInfo.timerMinVal = .01;
    devInfo.timerStep = .01;
    devInfo.suppAcqModes = ACQMODE_ACQSTART_TIMERSTOP;
    devInfo.suppCallback = FALSE;
    devInfo.maxPulseCount = 10000;
    devInfo.maxPulseHeight = 1.2;
    devInfo.maxPulsePeriod = 1e-5;
    devInfo.extDacMaxV = 5.0;
    devInfo.extDacMinV = 0;
    devInfo.extDacStep = 5.0/4095;
    _snprintf(devInfo.chipboardID, MPX_MAX_CHBID, "%d", 2000 + id);
    initHwInfoItems();

    imageFileName[0] = '\0';
    imageData = new i16[MATRIX_SIZE];
    memset(imageData, 0, MATRIX_SIZE * sizeof(i16));
}

HwDummy::~HwDummy()
{
    delete[] imageData;
}

void HwDummy::initHwInfoItems()
{
    // 0
    int i = 0;
    hwInfoItems[i].count = 1;
    hwInfoItems[i].data = &version;
    hwInfoItems[i].name = "Version";
    hwInfoItems[i].descr = "Dummy HW version";
    hwInfoItems[i].flags = 0;
    hwInfoItems[i].type = TYPE_U32;
    i++;

    // 1
    hwInfoItems[i].count = 1;
    hwInfoItems[i].data = &addNoise;
    hwInfoItems[i].name = "Add noise";
    hwInfoItems[i].descr = "Add noise";
    hwInfoItems[i].flags = MPX_HWINFO_CFGSAVE | MPX_HWINFO_CANCHANGE | MPX_HWINFO_MPXFRAME;
    hwInfoItems[i].type = TYPE_BOOL;
    i++;

    // 2
    hwInfoItems[i].count = MPX_MAX_PATH;
    hwInfoItems[i].data = imageFileName;
    hwInfoItems[i].name = "Data file name";
    hwInfoItems[i].descr = "Data file name for image";
    hwInfoItems[i].flags = MPX_HWINFO_CFGSAVE | MPX_HWINFO_CANCHANGE;
    hwInfoItems[i].type = TYPE_CHAR;
    i++;
}

int HwDummy::reinit()
{
    return initData();
}

int HwDummy::initData()
{
    if (imageFileName[0] == '\0')
        devMgr.getDefImageFileName(imageFileName);
    int rc = initImage();
    if (rc)
        memset(imageData, 0, MATRIX_SIZE * sizeof(i16));
    return rc;
}

int HwDummy::initImage()
{
    FILE *file = fopen(imageFileName, "r");
    if (!file)
    {
        _snprintf(lastErrStr, MPXCTRL_MAX_MSGLENGTH, "Cannot open file \"%s\" for reading: %s.", imageFileName, strerror(errno));
        return MPXERR_FILE_OPENREAD;
    }

    i16 *pitem = imageData;
    u32 i;
    for (i = 0; i < MATRIX_SIZE; i++)
    {
        if (fscanf(file, "%hd", pitem++) != 1)
            break;
    }

    if (i != MATRIX_SIZE)
    {
        if (ferror(file))
            _snprintf(lastErrStr, MPXCTRL_MAX_MSGLENGTH, "Error occurred while reading from file %s: %s", imageFileName, strerror(errno));
        else
            _snprintf(lastErrStr, MPXCTRL_MAX_MSGLENGTH, "Reading %u items from file %s failed", MATRIX_SIZE, imageFileName);
        fclose(file);
        return MPXERR_FILE_READ;
    }

    fclose(file);
    return 0;
}

#define CHECK_HWINFOINDEX(index)\
    if (index < 0 || index >= hwItemCount) {\
        _snprintf(lastErrStr, MPXCTRL_MAX_MSGLENGTH, "Invalid index (%d) of hw specific info item.", index);\
        return MPXERR_INVALID_PARVAL; \
    }

int HwDummy::getHwInfo(int index, HwInfoItem *hwInfo, int *dataSize)
{
    CHECK_HWINFOINDEX(index);
    HwInfoItem &locInfo = hwInfoItems[index];
    hwInfo->type = locInfo.type;
    hwInfo->count = locInfo.count;
    hwInfo->flags = locInfo.flags;
    hwInfo->name = locInfo.name;
    hwInfo->descr = locInfo.descr;

    int requiredSize = sizeofType[locInfo.type] * locInfo.count;
    if (*dataSize < requiredSize)
    {
        *dataSize = requiredSize;
        strcpy(lastErrStr, "Invalid size of buffer for hw specific info item data.");
        return MPXERR_BUFFER_SMALL;
    }
    memcpy(hwInfo->data, locInfo.data, requiredSize);
    return 0;
}

int HwDummy::getHwInfoFlags(int index, u32 *flags)
{
    CHECK_HWINFOINDEX(index);
    *flags = hwInfoItems[index].flags;
    return 0;
}

int HwDummy::setHwInfo(int index, void *data, int dataSize)
{
    CHECK_HWINFOINDEX(index);
    HwInfoItem &info = hwInfoItems[index];
    if ((u32) dataSize != sizeofType[info.type] * info.count)
    {
        strcpy(lastErrStr, "Invalid size of data for hw specific info item.");
        return MPXERR_INVALID_PARVAL;
    }

    if (info.flags & MPX_HWINFO_CANCHANGE)
    {
        memcpy(info.data, data, dataSize);
        if (index == 2)
            return initData();
    }
    return 0;
}

int HwDummy::getDevInfo(DevInfo *devInfoOut)
{
    memcpy(devInfoOut, &devInfo, sizeof(DevInfo));
    return 0;
}

int HwDummy::setAcqPars(AcqParams *pars)
{
    if (pars->mode & (ACQMODE_ACQSTART_HWTRIGSTOP | ACQMODE_HWTRIGSTART_TIMERSTOP | ACQMODE_HWTRIGSTART_HWTRIGSTOP))
    {
        strncpy(lastErrStr, "Invalid acq mode.", MPXCTRL_MAX_MSGLENGTH);
        return MPXERR_INVALID_PARVAL;
    }
    memcpy(&acqPars, pars, sizeof(AcqParams));
    return 0;
}

int HwDummy::startAcquisition()
{
    elapsedTime = -1;
    startTime = getPreciseTime();
    return 0;
}

int HwDummy::stopAcquisition()
{
    if (elapsedTime < 0)
        elapsedTime = getPreciseTime() - startTime;
    return 0;
}

int HwDummy::testPulses(double pulseHeight, double period, u32 count)
{
    return 0;
}

int HwDummy::setMpxDACs(DACTYPE dacVals[], int size, byte senseChip, byte extDacChip, int codes[], u32 tpReg)
{
    return 0;
}

int HwDummy::getMpxDacVal(int chipNumber, double *value)
{
    *value = 1;
    return 0;
}

int HwDummy::setPixelsCfg(byte cfgs[])
{
    return 0;
}

int HwDummy::resetMatrix()
{
    return 0;
}

int HwDummy::readMatrix(i16 *buff, u32 buffSize)
{
    memcpy(buff, imageData, buffSize*sizeof(i16));
    if (addNoise)
    {
        srand((unsigned)time(0));
        for (int i = 0; i < MATRIX_SIZE; i++)
            buff[i] = (buff[i] + (rand() % 1000)) % 8000;
    }
    return 0;
}

int HwDummy::writeMatrix(i16 *buff, u32 buffSize)
{
    return 0;
}

int HwDummy::generalReset()
{
    return 0;
}

int HwDummy::checkBusy(BOOL *busy)
{
    if (getPreciseTime() - startTime > acqPars.time)
        *busy = FALSE;
    else
        *busy = TRUE;
    return 0;
}

double HwDummy::getPreciseTime()
{
    static LARGE_INTEGER perfFrequency = {0};
    LARGE_INTEGER currentCount;
    if (perfFrequency.QuadPart == 0)
        QueryPerformanceFrequency(&perfFrequency);
    QueryPerformanceCounter(&currentCount);
    return currentCount.QuadPart / (double) perfFrequency.QuadPart;
}
